@@ -14,45 +14,19 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
 
     this.init = function() {
         this.gBrowser = gBrowser;
+        this.isPrivate = isWindowPrivate();
+        this.prefs = NSQ.prefs;
         this.updateZoomMenu();
 
         this.observer = new NSQ.interfaces.Observer();
-        this.observer.watcher = {
-            onEnterPrivateBrowsing: function() {
-                this.closeSiteSettings();
-                // Switching the private browsing mode.  Store any current pending
-                // changes now.
-                NSQ.prefs.saveSiteList(true);
-                // Save current (non-private) site data for when we exit private
-                // browsing.
-                this.origSites = NSQ.prefs.cloneSites();
-            },
 
-            onExitPrivateBrowsing: function() {
-                this.closeSiteSettings();
-                // Restore previously saved site data and rezoom/style all tabs.
-                NSQ.prefs.sites = this.origSites;
-                this.origSites = null;
-                NSQ.browser.zoomAll();
-                NSQ.browser.styleAll();
-            },
-
-            closeSiteSettings: function() {
-                if (NSQ.storage.dialogs.site)
-                    NSQ.storage.dialogs.site.die();
-            }
-        };
-
-        if (this.observer.inPrivateBrowsing)
-            this.observer.watcher.onEnterPrivateBrowsing();
-
-        window.addEventListener('DOMMouseScroll', this.handleMouseScroll, false); 
+        window.addEventListener('DOMMouseScroll', this.handleMouseScroll, false);
         // XXX: used for image zoom, which feature is currently removed.
         //window.addEventListener("resize", this.handleResize, false);
         gBrowser.tabContainer.addEventListener('TabOpen', this.handleTabOpen, false);
         gBrowser.tabContainer.addEventListener('TabSelect', this.handleTabSelect, false);
         gBrowser.tabContainer.addEventListener('TabClose', this.handleTabClose, false);
-        
+
         this.zoomAll(null, true);
         this.styleAll(null);
 
@@ -77,12 +51,12 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
         gBrowser.tabContainer.removeEventListener('TabOpen', this.handleTabOpen, false);
         gBrowser.tabContainer.removeEventListener('TabSelect', this.handleTabSelect, false);
         gBrowser.tabContainer.removeEventListener('TabClose', this.handleTabClose, false);
-        window.removeEventListener('DOMMouseScroll', this.handleMouseScroll, false); 
+        window.removeEventListener('DOMMouseScroll', this.handleMouseScroll, false);
     };
 
     this.hookZoomButtonsForReset = function() {
         if ($('zoom-out-button')) {
-            $('zoom-out-button').onclick = $('zoom-in-button').onclick = 
+            $('zoom-out-button').onclick = $('zoom-in-button').onclick =
                 function(event) {
                     if (event.button == 1)
                         NoSquint.cmd.buttonReset(event);
@@ -91,7 +65,7 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
             /* TODO
             $('zoom-out-button').addEventListener('DOMMouseScroll', function(event) {
                 // Implement wheel zooming over button here.
-            }, false); 
+            }, false);
             */
         }
     };
@@ -189,7 +163,7 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
             text = full = false;
             increment = NSQ.prefs.zoomIncrement * (event.detail < 0 ? 1 : -1);
             img = isImage(browser);
-                
+
             if (NSQ.prefs.wheelZoomInvert)
                 increment *= -1;
 
@@ -390,10 +364,18 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
             browser.getUserData('nosquint').site = site;
             debug('getZoomForBrowser(): after getSiteFromBrowser(), site=' + site);
         }
-
-        var [text, full] = NSQ.prefs.getZoomForSite(site);
-        var [text_default, full_default] = NSQ.prefs.getZoomDefaults(site);
-        return [text || text_default, full || full_default];
+        // If the site dialog is open on the current site, get the zoom levels
+        // from that instead as it should be treated as authoritative.
+        var siteDlg = NSQ.storage.dialogs.site;
+        if (siteDlg && siteDlg.site == site) {
+            // Exception: if the site dialog is for a private window other
+            // than this one, don't use the site dialog settings otherwise
+            // private window settings would leak out of the window.
+            if (this.isPrivate == isWindowPrivate(siteDlg.browser.contentWindow) &&
+                (!this.isPrivate || siteDlg.browser == browser))
+                return siteDlg.getZoomLevels();
+        }
+        return NSQ.prefs.getZoomForSiteWithDefaults(site);
     };
 
 
@@ -414,7 +396,7 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
 
     this.attach = function(browser) {
         var listener = new NSQ.interfaces.ProgressListener(browser);
-        browser.addProgressListener(listener, CI.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
+        browser.addProgressListener(listener);
         debug('attach(): attached browser URI=' + browser.docShell.document.URL);
 
         var userData = {
@@ -492,7 +474,7 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
         zoomAllTimer = null;
     };
 
-    /* Queues a zoomAll.  Useful when we might otherwise call zoomAll() 
+    /* Queues a zoomAll.  Useful when we might otherwise call zoomAll()
      * multiple times, such as in the case of multiple preferences being
      * updated at once.
      */
@@ -508,8 +490,15 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
      */
     this.getStyleForBrowser = function(browser) {
         var site = browser.getUserData('nosquint').site;
-        var style = NSQ.prefs.getStyleForSite(site);
-        return NSQ.prefs.applyStyleGlobals(style);
+        // Use site dialog settings if applicable.  See getZoomForBrowser()
+        // for explanation.
+        var siteDlg = NSQ.storage.dialogs.site;
+        if (siteDlg && siteDlg.site == site) {
+            if (this.isPrivate == isWindowPrivate(siteDlg.browser.contentWindow) &&
+                (!this.isPrivate || siteDlg.browser == browser))
+                return siteDlg.getStyle();
+        }
+        return NSQ.prefs.getStyleForSiteWithDefaults(site);
     };
 
     /* Returns CSS string for the given style object.
@@ -587,8 +576,18 @@ NoSquint.browser = NoSquint.ns(function() { with (NoSquint) {
             stylers.push(this.getDocumentStyler(browser, doc));
 
         debug("style(): num stylers=" + stylers.length);
-        for (let styler in iter(stylers))
-            styler(style);
+        for (let i = stylers.length - 1; i >= 0; i--) {
+            // A styler may raise if it applies to a dead object (which
+            // happens if e.g. a DOM frame is dynamically removed).  If it
+            // fails, remove it from the list of active stylers for this
+            // browser.
+            try {
+                stylers[i](style);
+            } catch (e) {
+                debug("Failed to apply style: " + e);
+                stylers.splice(i, 1);
+            }
+        }
 
         if (browser == gBrowser.selectedBrowser)
             this.queueUpdateStatus();
