@@ -1,18 +1,25 @@
 // chrome://browser/content/browser.xul
 
 /******************************************************************************
- * Preferences (Singleton)
+ * Preferences
  *
  * Namespace for anything pref related, including service objects, any
  * currently cached values, routines for parsing, or convenience functions
  * for accessing preferences.
+ *
+ * Each Private Browsing window gets its own NSQ.prefs which doesn't listen
+ * to sites pref changes.  Non-private windows share the same NSQ.prefs.
  */
 
 NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
-    // Namespace is a singleton, so return any previously instantiated prefs object.
-    if (NSQ.storage.prefs)
-        return NSQ.storage.prefs;
-    NSQ.storage.prefs = this;
+    this.isPrivate = isWindowPrivate();
+    if (!this.isPrivate) {
+        // For non-private windows, namespace is a singleton, so return any
+        // previously instantiated prefs object.
+        if (NSQ.storage.prefs)
+            return NSQ.storage.prefs;
+        NSQ.storage.prefs = this;
+    }
 
     this.id = 'NoSquint.prefs';
     this.defaultColors = {
@@ -24,7 +31,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
 
     /* Active window we can use for window methods (e.g. setTimeout).  Because
      * NSQ.prefs is a singleton, it could be that the window we initialized
-     * with has been closed.  In that case, setTimeout will fail with 
+     * with has been closed.  In that case, setTimeout will fail with
      * NS_ERROR_NOT_INITIALIZED.  So we keep a reference to an available
      * window here we can call window.* methods with, and if the window
      * goes away, we find a new one using foreachNSQ().
@@ -130,7 +137,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
      * may not actually still be alive.
      */
     this.winFunc = function(func) {
-        var args = Array.prototype.slice.call(arguments, 1); 
+        var args = Array.prototype.slice.call(arguments, 1);
         try {
             return this.window[func].apply(this.window, args);
         } catch (e) {
@@ -145,12 +152,20 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
     };
 
     this.preload = function() {
-        // Initialize preferences in this order; some of them require other prefs
-        // have been loaded.  (e.g. forgetMonths needs rememberSites)
+        // If this is a prefs instance for a private window, clone the shared
+        // non-private site list if available.
+        if (this.isPrivate && NSQ.storage.prefs)
+            this.sites = NSQ.storage.prefs.cloneSites();
+        else
+            this.observe(null, "nsPref:changed", "sites");
+
+        // Initialize other preferences in this order; some of them require
+        // other prefs have been loaded.  (e.g. forgetMonths needs
+        // rememberSites)
         var prefs = [
             'fullZoomLevel', 'textZoomLevel', 'zoomIncrement', 'wheelZoomEnabled', 'hideStatus',
-            'action', 'sitesSaveDelay', 'rememberSites', 'exceptions', 'sites', 'forgetMonths',
-            'fullZoomPrimary', 'wheelZoomInvert', 'zoomImages', 'colorText', 'colorBackground', 
+            'action', 'sitesSaveDelay', 'rememberSites', 'exceptions', 'forgetMonths',
+            'fullZoomPrimary', 'wheelZoomInvert', 'zoomImages', 'colorText', 'colorBackground',
             'colorBackgroundImages', 'linksUnvisited', 'linksVisited', 'linksUnderline'
         ];
         for (let pref in iter(prefs))
@@ -164,7 +179,6 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
             // Not a pref change.
             return;
 
-        debug('observe(): data=' + data);
         switch (data) {
             case 'siteSpecific':
                 if (branchBZ.getBoolPref('siteSpecific') == false || NSQ.storage.disabled || NSQ.storage.quitting)
@@ -225,9 +239,10 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
                 foreachNSQ(function(NSQ) {
                     if (NSQ.browser) {
                         $('nosquint-status').hidden = hideStatus;
-                        if (!hideStatus)
+                        if (!hideStatus) {
                             // Status now being shown; update it to reflect current values.
                             NSQ.browser.queueUpdateStatus();
+                        }
                     }
                 });
                 break;
@@ -261,6 +276,11 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
                     ignoreNextSitesChange = false;
                     break;
                 }
+                if (subject != null && this.isPrivate)
+                    // Private windows don't refresh from pref changes.  (If subject is
+                    // null then this is a preload() and we don't want to skip that.)
+                    break;
+
                 this.sites = this.parseSites(branchNS.getCharPref('sites'));
                 if (saveTimer) {
                     /* FIXME: looks like the sites list pref was updated (possibly by
@@ -319,7 +339,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
      */
     this.parseSites = function(sitesStr) {
         /* Parse site list from prefs.  The prefs string a list of site specs,
-         * delimited by a space, in the form: 
+         * delimited by a space, in the form:
          *
          *     sitename=text_level,timestamp,visits,full_level,textcolor,bgcolor,
          *              nobgimages,linkunvis,linkvis,linkunderline
@@ -389,7 +409,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
          */
         function regexpify(pattern, re_star, re_dblstar) {
             var parts = pattern.split(/(\[\*+\]|\*+)/);
-            var pattern = [];
+            pattern = [];
             var sub = [];
             var length = 0;
 
@@ -405,7 +425,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
             for (let part in iter(parts)) {
                 if (part == '')
                     continue;
-                if (wildcards[part]) 
+                if (wildcards[part])
                     pattern.push(wildcards[part]);
                 else {
                     length += part.length;
@@ -449,7 +469,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
     this.pruneSites = function()  {
         if (!this.rememberSites || this.forgetMonths == 0)
             return;
-    
+
         var remove = [];
         var now = new Date();
         for (let [site, settings] in items(this.sites)) {
@@ -473,6 +493,12 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
             pruneTimer = this.winFunc('setTimeout', function() { pruneTimer = null; NSQ.prefs.pruneSites(); }, 24*60*60*1000);
     };
 
+    this.isSiteRecordDefault = function(record) {
+        if ([record[0]].concat(record.slice(3)).toString() == [0, 0, '0', '0', false, '0', '0', false].toString())
+            return true;
+        else
+            return false;
+    }
 
     /* Updates the site list for the given site name to set the given levels
      * (2-tuple of [text, full]), and then queues a site list save.
@@ -509,14 +535,11 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
 
         // Check newly updated record against defaults.  If all values are default, we
         // remove the record.
-        if ([record[0]].concat(record.slice(3)).toString() == [0, 0, '0', '0', false, '0', '0', false].toString())
-            // All defaults.
+        if (this.isSiteRecordDefault(record))
             delete this.sites[site];
 
         debug('updateSiteList(): site=' + site + ', record=' + record);
-
-        if (this.rememberSites)
-            this.queueSaveSiteList();
+        this.queueSaveSiteList(true);
     };
 
 
@@ -526,14 +549,16 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
     this.updateSiteTimestamp = function(site) {
         if (!site || !this.sites[site])
             return;
-
-        this.sites[site][1] = new Date().getTime();
-        this.sites[site][2] += 1;
-        if (this.rememberSites)
-            // Save updated timestamp.  Timestamps are only updated on
-            // the first page accessed for a given visit to that site,
-            // so this shouldn't be too bad.
-            this.queueSaveSiteList();
+        if (this.isSiteRecordDefault(this.sites[site]))
+            delete this.sites[site];
+        else {
+            this.sites[site][1] = new Date().getTime();
+            this.sites[site][2] += 1;
+        }
+        // Save updated timestamp.  Timestamps are only updated on
+        // the first page accessed for a given visit to that site,
+        // so this shouldn't be too bad.
+        this.queueSaveSiteList(false);
     };
 
 
@@ -543,7 +568,11 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
      * else the next time a change is made in the Settings dialog, it will
      * be ignored.
      */
-    this.queueSaveSiteList = function() {
+    this.queueSaveSiteList = function(flush) {
+        if (!this.rememberSites || this.isPrivate)
+            // Now allowed to save changes, don't queue save.
+            return;
+
         this.stopQueueSaveSiteList();
 
         /* The list is actually saved (by default) 5s later, so if the user
@@ -551,13 +580,14 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
          * needlessly iterating over the sites array.
          */
         debug("queueSaveSiteList(): delay=" + this.saveDelay);
-        saveTimer = this.winFunc('setTimeout', function() NSQ.prefs.saveSiteList(), this.saveDelay);
+        saveTimer = this.winFunc('setTimeout', function() NSQ.prefs.saveSiteList(flush), this.saveDelay);
     };
 
-
-    /* Store the sites list right now. */
-    this.saveSiteList = function(force) {
-        if (!this.rememberSites || (NSQ.browser && NSQ.browser.observer.inPrivateBrowsing && !force))
+    /* Store the sites list right now.  If flush is true, the prefs file is
+     * committed to disk.
+     */
+    this.saveSiteList = function(flush) {
+        if (!this.rememberSites || this.isPrivate)
             /* Private Browsing mode is enabled or rememberSites disabled; do
              * not save site list.
              */
@@ -577,13 +607,13 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
          */
         ignoreNextSitesChange = true;
         branchNS.setCharPref('sites', sites.join(' '));
-        this.save();
-        debug("saveSiteList(): took: " + (new Date().getTime() - t0) + "ms");
+        if (flush)
+            this.save();
+        debug("saveSiteList(): took: " + (new Date().getTime() - t0) + "ms, flush=" + flush);
         this.winFunc('clearTimeout', saveTimer);
         saveTimer = null;
     };
 
-
     /* Stops a previously queued site list save.  Returns true if a save was
      * queued and aborted, or false if no save was queued.
      */
@@ -654,7 +684,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
         uri_host += ':' + uri_port;
 
         var match = null;
-        
+
         /* Iterate over each exception, trying to match it with the URI.
          * We break the loop on the first match, because exceptions are
          * sorted with highest weights first.
@@ -698,6 +728,13 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
         return [null, null];
     };
 
+
+    this.getZoomForSiteWithDefaults = function(site) {
+        var [text, full] = this.getZoomForSite(site);
+        var [text_default, full_default] = this.getZoomDefaults(site);
+        return [text || text_default, full || full_default];
+    };
+
     /* Gets the style parameters for the given site name.  Returns null if
      * the site has no settings.
      */
@@ -716,6 +753,11 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
         return null;
     };
 
+   this.getStyleForSiteWithDefaults = function(site) {
+        var style = this.getStyleForSite(site);
+        return this.applyStyleGlobals(style);
+    };
+
     /* Applies global styles to the given style object.  Attributes that have
      * no site-local or global value are null.
      */
@@ -731,7 +773,7 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
     };
 
 
-    // Saves all preferences, including exceptions BUT NOT sites.
+    // Saves all global preferences, including exceptions BUT NOT sites.
     this.saveAll = function(exceptions) {
         const intPrefs = [
             'fullZoomLevel', 'textZoomLevel', 'zoomIncrement', 'forgetMonths'
@@ -841,8 +883,11 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
                 cur_parts = addon.version.split('.');
                 last_parts = (NSQ.prefs.lastVersion || '0.0').split('.');
                 if (cur_parts[0] > last_parts[0] || (cur_parts[0] == last_parts[0] && cur_parts[1] > last_parts[1]))
-                    if (!found)
-                        NSQ.browser.gBrowser.selectedTab = gBrowser.addTab('about:nosquint');
+                    if (!found) {
+                        defer(500, function() {
+                            NSQ.browser.gBrowser.selectedTab = gBrowser.addTab('about:nosquint');
+                        });
+                    }
                 branchNS.setCharPref('version', addon.version);
                 NoSquint.prefs.save();
             }
@@ -858,6 +903,5 @@ NoSquint.prefs = NoSquint.ns(function() { with(NoSquint) {
             AddonManager.getAddonByID('nosquint@urandom.ca', callback);
         }
     };
-
-
+    return this;
 }});
